@@ -23,21 +23,21 @@
 #' @importFrom stats model.matrix model.response rnorm delete.response terms window ts end glm poisson
 #' @rdname walker
 #' @useDynLib walker, .registration = TRUE
-#' @param formula An object of class \code{\link[stats]{formula}} with additional terms 
+#' @param formula An object of class \code{{formula}} with additional terms 
 #' \code{rw1} and/or \code{rw2} e.g. \code{y ~ x1 + rw1(~ -1 + x2)}. See details.
-#' @param data An optional data.frame or object coercible to such, as in \code{\link[stats]{lm}}.
+#' @param data An optional data.frame or object coercible to such, as in \code{{lm}}.
 #' @param beta_prior A length vector of length two which defines the 
 #' prior mean and standard deviation of the Gaussian prior for time-invariant coefficients
 #' @param sigma_y_prior A vector of length two, defining the truncated Gaussian prior for 
 #' the observation level standard deviation. Not used in \code{walker_glm}. 
 #' @param chains Number of Markov chains. Default is 4.
-#' @param init Initial value specification, see \code{\link[rstan]{sampling}}. 
+#' @param init Initial value specification, see \code{\link{sampling}}. 
 #' Note that compared to default in \code{rstan}, here the default is a to sample from the priors.
 #' @param return_x_reg If \code{TRUE}, does not perform sampling, but instead returns the matrix of 
 #' predictors after processing the \code{formula}.
-#' @param gamma An optional vector defining a damping of the random walk noises. More specifically, 
-#' the variance of the conditional distribution of state_t+1 given state is of form gamma_t^2 * sigma^s2.
-#' @param ... Further arguments to \code{\link[rstan]{sampling}}.
+#' @param gamma_y An optional vector defining a damping of the observational level noise. 
+#' More specifically, \eqn{\sigma_t = gamma_t * \sigma_y}.
+#' @param ... Further arguments to \code{\link{sampling}}.
 #' @return A list containing the \code{stanfit} object, observations \code{y},
 #'   and covariates \code{xreg} and \code{xreg_new}.
 #' @seealso \code{\link{walker_glm}} for non-Gaussian models.
@@ -51,7 +51,7 @@
 #'     beta_prior = c(1000, 100), 
 #'     sigma_prior = c(0, 100)), 
 #'   sigma_y_prior = c(0, 100), 
-#'   iter = 50, chains = 1)
+#'   iter = 200, chains = 1)
 #'   
 #' rw2_fit <- walker(Nile ~ -1 + 
 #'   rw2(~ 1,
@@ -59,7 +59,7 @@
 #'     sigma_prior = c(0, 100), 
 #'     slope_prior = c(0, 100)), 
 #'   sigma_y_prior = c(0, 100), 
-#'   iter = 50, chains = 1)
+#'   iter = 200, chains = 1)
 #'   
 #' g_y <- geom_point(data = data.frame(y = Nile, x = time(Nile)), 
 #'   aes(x, y, alpha = 0.5), inherit.aes = FALSE) 
@@ -79,7 +79,7 @@
 #' dat <- data.frame(y, cos_t, sin_t)
 #' fit <- walker(y ~ -1 + 
 #'   rw1(~ cos_t + sin_t, beta_prior = c(0, 10), sigma_prior = c(0, 2)), 
-#'   sigma_y_prior = c(0, 10), data = dat, chains = 1, iter = 250)
+#'   sigma_y_prior = c(0, 10), data = dat, chains = 1, iter = 500)
 #' print(fit$stanfit, pars = c("sigma_y", "sigma_rw1"))
 #' 
 #' plot_coefs(fit)
@@ -94,7 +94,7 @@
 #' }
 #' 
 walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
-  return_x_reg = FALSE, gamma = NULL, ...) {
+  return_x_reg = FALSE, gamma_y = NULL, ...) {
   
   if (missing(data)) data <- environment(formula)
   # Modifying formula object, catching special functions
@@ -131,23 +131,32 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
     comp <- vars[[1 + attr(all_terms, "specials")$rw1[1]]]
     rw1_out <- eval(comp, envir = data, enclos = parent.frame())
     # only intercept
-    if (nrow(rw1_out$xreg) == 0) rw1_out$xreg <- matrix(1, n, 1)
+    if (nrow(rw1_out$xreg) == 0) {
+      rw1_out$xreg <- matrix(1, n, 1)
+      rw1_out$gamma <- matrix(1, 1, n)
+    }
+    if (nrow(rw1_out$xreg) != n) stop("length of the series and covariates do not match.")
   } else {
     rw1_out <- list(xreg = matrix(0, n, 0), 
-      beta_prior = numeric(2), sigma_prior = numeric(2))
+      beta_prior = numeric(2), sigma_prior = numeric(2), 
+      gamma = matrix(0, 0, n))
   }
   if (!is.null(attr(all_terms, "specials")$rw2)) {
     comp <- vars[[1 + attr(all_terms, "specials")$rw2[1]]]
     rw2_out <- eval(comp, envir = data, enclos = parent.frame())
     # only intercept
-    if (nrow(rw2_out$xreg) == 0) rw2_out$xreg <- matrix(1, n, 1)
+    if (nrow(rw2_out$xreg) == 0) {
+      rw2_out$xreg <- matrix(1, n, 1)
+      rw2_out$gamma <- matrix(1, 1, n)
+    }
+    if (nrow(rw2_out$xreg) != n) stop("length of the series and covariates do not match.")
   } else {
     rw2_out <- list(xreg = matrix(0, n, 0), 
-      beta_prior = numeric(2), sigma_prior = numeric(2), slope_prior = numeric(2))
+      beta_prior = numeric(2), sigma_prior = numeric(2), 
+      slope_prior = numeric(2), gamma = matrix(0, 0, n))
   }
   
   xreg_rw <- cbind(rw1_out$xreg, rw2_out$xreg)
-  
   k_fixed <- max(0, ncol(xreg_fixed))
   k_rw1 <- max(0, ncol(rw1_out$xreg))
   k_rw2 <- max(0, ncol(rw2_out$xreg))
@@ -163,13 +172,13 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
     stop("sigma_prior should be should be a vector of length two, defining the mean and standard deviation for the Gaussian prior of the standard deviation of y. ")
   }
   
-  if (is.null(gamma)) {
-    gamma <- rep(1, n) 
+  if (is.null(gamma_y)) {
+    gamma_y <- rep(1, n) 
   } else {
-    if (length(gamma) != n) 
+    if (length(gamma_y) != n) 
       stop("The length of gamma vector should equal to the number of observations. ")
-    if (!is.numeric(gamma) | any(gamma < 0 | is.na(gamma))) 
-      stop("Argument 'gamma' should be numeric vector of nonnegative values. ")
+    if (!is.numeric(gamma_y) | any(gamma_y < 0 | is.na(gamma_y))) 
+      stop("Argument 'gamma_y' should be numeric vector of nonnegative values. ")
   } 
   
   stan_data <- list(
@@ -182,6 +191,7 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
     xreg_fixed = xreg_fixed, 
     xreg_rw = t(xreg_rw), 
     y = y, 
+    y_miss = as.integer(is.na(y)),
     sigma_y_mean = sigma_y_prior[1],
     sigma_y_sd = sigma_y_prior[2],
     beta_fixed_mean = if (k_fixed > 0) beta_prior[1] else 0,
@@ -195,8 +205,12 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
     sigma_rw2_mean = rw2_out$sigma_prior[1],
     sigma_rw2_sd = rw2_out$sigma_prior[2],
     slope_mean = rw2_out$slope_prior[1],
-    slope_sd = rw2_out$slope_prior[2]
+    slope_sd = rw2_out$slope_prior[2],
+    gamma_y = gamma_y,
+    gamma_rw1 = rw1_out$gamma,
+    gamma_rw2 = rw2_out$gamma
   )
+  stan_data$y[is.na(y)] <- 0 ## Stan does not accept NA's
   
   if (missing(chains)) chains <- 4
   if (missing(init)) {
@@ -267,7 +281,7 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
 #' \code{"glm"} (mode is obtained from time-invariant GLM), \code{"mle"} 
 #' (default; mode is obtained from maximum likelihood estimate of the model), 
 #' or numeric vector (custom guess).
-#' @param u For Poisson model, a vector of exposures i.e. E(y) = u*exp(x*beta). 
+#' @param u For Poisson model, a vector of exposures i.e. \eqn{E(y) = u*exp(x*beta)}. 
 #' For binomial, a vector containing the number of trials. Defaults 1.
 #' @param mc_sim Number of samples used in importance sampling. Default is 50.
 #' @return A list containing the \code{stanfit} object, observations \code{y},
@@ -277,16 +291,16 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
 #' @export
 #' @examples 
 #' 
-#' ## note very low number of iterations for the CRAN checks
+#' \dontrun{
 #' 
 #' data("discoveries", package = "datasets")
 #' out <- walker_glm(discoveries ~ -1 + 
 #'   rw2(~ 1, beta_prior = c(0, 10), sigma_prior = c(0, 2), slope_prior = c(0, 2)), 
-#'   distribution = "poisson", iter = 50, chains = 1, refresh = 0)
+#'   distribution = "poisson", iter = 1000, chains = 1, refresh = 0)
 #' 
 #' plot_fit(out)
 #' 
-#' \dontrun{
+#' 
 #' 
 #' set.seed(1)
 #' n <- 25
@@ -300,7 +314,7 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
 #' 
 #' out <- walker_glm(y ~ -1 + rw1(~ x, beta_prior = c(0, 10), 
 #'   sigma_prior = c(0, 10)), distribution = "poisson", 
-#'   iter = 250, chains = 1, refresh = 0)
+#'   iter = 1000, chains = 1, refresh = 0)
 #' print(out$stanfit, pars = "sigma_rw1") ## approximate results
 #' if (require("diagis")) {
 #'   weighted_mean(extract(out$stanfit, pars = "sigma_rw1")$sigma_rw1, 
@@ -313,7 +327,7 @@ walker <- function(formula, data, sigma_y_prior, beta_prior, init, chains,
 #'              
 walker_glm <- function(formula, data, beta_prior, init, chains,
   return_x_reg = FALSE, distribution ,
-  initial_mode = "kfas", u, mc_sim = 50, gamma = NULL, ...) {
+  initial_mode = "kfas", u, mc_sim = 50, ...) {
   
   distribution <- match.arg(distribution, choices = c("poisson", "binomial"))
   
@@ -352,19 +366,29 @@ walker_glm <- function(formula, data, beta_prior, init, chains,
     comp <- vars[[1 + attr(all_terms, "specials")$rw1[1]]]
     rw1_out <- eval(comp, envir = data, enclos = parent.frame())
     # only intercept
-    if (nrow(rw1_out$xreg) == 0) rw1_out$xreg <- matrix(1, n, 1)
+    if (nrow(rw1_out$xreg) == 0) {
+      rw1_out$xreg <- matrix(1, n, 1)
+      rw1_out$gamma <- matrix(1, 1, n)
+    }
+    if (nrow(rw1_out$xreg) != n) stop("length of the series and covariates do not match.")
   } else {
     rw1_out <- list(xreg = matrix(0, n, 0), 
-      beta_prior = numeric(2), sigma_prior = numeric(2))
+      beta_prior = numeric(2), sigma_prior = numeric(2), 
+      gamma = matrix(0, 0, n))
   }
   if (!is.null(attr(all_terms, "specials")$rw2)) {
     comp <- vars[[1 + attr(all_terms, "specials")$rw2[1]]]
     rw2_out <- eval(comp, envir = data, enclos = parent.frame())
     # only intercept
-    if (nrow(rw2_out$xreg) == 0) rw2_out$xreg <- matrix(1, n, 1)
+    if (nrow(rw2_out$xreg) == 0) {
+      rw2_out$xreg <- matrix(1, n, 1)
+      rw2_out$gamma <- matrix(1, 1, n)
+    }
+    if (nrow(rw2_out$xreg) != n) stop("length of the series and covariates do not match.")
   } else {
     rw2_out <- list(xreg = matrix(0, n, 0), 
-      beta_prior = numeric(2), sigma_prior = numeric(2), slope_prior = numeric(2))
+      beta_prior = numeric(2), sigma_prior = numeric(2), 
+      slope_prior = numeric(2), gamma = matrix(0, 0, n))
   }
   
   xreg_rw <- cbind(rw1_out$xreg, rw2_out$xreg)
@@ -375,7 +399,6 @@ walker_glm <- function(formula, data, beta_prior, init, chains,
   if (return_x_reg) return(list(xreg_fixed = xreg_fixed, xreg_rw = xreg_rw))
   
   if (any(is.na(xreg_fixed)) || any(is.na(xreg_rw))) stop("Missing values in covariates are not allowed.")
-  if (any(is.na(y))) stop("Missing values in response are not (yet) allowed.")
   
   if(k_fixed > 0 && length(beta_prior) != 2) {
     stop("beta_prior should be a vector of length two, defining the mean and standard deviation for the Gaussian prior of fixed coefficients. ")
@@ -443,14 +466,7 @@ walker_glm <- function(formula, data, beta_prior, init, chains,
       stop("Argument 'initial_mode' should be either 'obs', 'glm', 'kfas', or a numeric vector.")
     )
   }
-  if (is.null(gamma)) {
-    gamma <- rep(1, n) 
-  } else {
-    if (length(gamma) != n) 
-      stop("The length of gamma vector should equal to the number of observations. ")
-    if (!is.numeric(gamma) | any(gamma < 0 | is.na(gamma))) 
-      stop("Argument 'gamma' should be numeric vector of nonnegative values. ")
-  } 
+
   stan_data <- list(
     k_fixed = k_fixed, 
     k_rw1 = k_rw1,
@@ -473,13 +489,17 @@ walker_glm <- function(formula, data, beta_prior, init, chains,
     slope_mean = slope_mean,
     slope_sd = slope_sd,
     y = pseudo_y, 
+    y_miss = as.integer(is.na(y)),
     Ht = pseudo_H, 
     y_original = y, 
     u = as.integer(u), 
     distribution = pmatch(distribution, c("poisson", "binomial")), 
     N = mc_sim,
-    gamma = gamma
+    gamma_rw1 = rw1_out$gamma,
+    gamma_rw2 = rw2_out$gamma
   )
+  stan_data$y[is.na(y)] <- 0 ## Stan does not accept NA's
+  stan_data$pseudo_y[is.na(y)] <- 0 ## Stan does not accept NA's
   
   if (missing(chains)) chains <- 4
   if (missing(init)) {
